/***************************************************************************
 *   copyright           : (C) 2002 by Hendrik Sattler                     *
 *   mail                : post@hendrik-sattler.de                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "common.h"
#include "helper.h"
#include "config.h"
#include "gtincl.h"
#include "w32compat.h"

#include <stdio.h>
#include <string.h>
#include <strings.h>
#include <ctype.h>

void help_print_line(const char* longopt,
		     const char* longopt_arg,
		     size_t longspace,
		     char shortoption,
		     const char* text)
{
  const char* temp;
  size_t line_len = 80;
  char line[6*80+1];
  char* lp; //pointer in line;
  size_t textspace;
  size_t shortspace = 4;
  size_t extraspace = 0;
  unsigned int i;

  longspace += 6;
  extraspace += (str_len(longopt)-str_width(longopt));
  extraspace += (str_len(longopt_arg)-str_width(longopt_arg));
  textspace = line_len-longspace-extraspace-shortspace-1;

  memset(line,0,sizeof(line));
  //first line of an option
  lp = line;
  if (longopt != NULL) {
    if (longopt_arg == NULL) snprintf(lp,longspace," --%s",longopt);
    else  snprintf(lp,longspace," --%s <%s>",longopt,longopt_arg);
  }
  memset(lp+strlen(lp),' ',sizeof(line)-1-strlen(lp));
  lp += longspace+extraspace;
  if (isalpha((int)shortoption)) {
    lp[0] = '-';
    lp[1] = shortoption;
  }
  lp += shortspace;

  do {
    if (str_width(text) <= textspace) {
      sprintf(lp,"%s",text);
      temp = NULL;
    } else {
      i = textspace;
      /* goto last valid multibyte sequence */
      while (strn_valid(text,i) == 0) --i;
      temp = text+i;
      /* goto last space */
      while (!isspace((int)*temp)) --temp;
      if (temp == text) temp = text+i;
      snprintf(lp,temp-text+1,"%s",text);
      memset(lp+(temp-text),0,1);
      while (isspace((int)*temp)) ++temp;
      text = temp;
    }
    fprintf(stdout,"%s\n",line);
    //other lines
    memset(line,' ',sizeof(line)-1);
    lp -= extraspace;
  } while (temp != NULL);
}

#ifndef PATH_SEPERATOR
#define PATH_SEPERATOR '/'
#endif

void help (char* me,
	   struct args_def_type* types,
	   struct args_def* params)
{
  char* name = strrchr(me,PATH_SEPERATOR);
  unsigned int i = 0, k;
  unsigned int total = 0;

  if (name == NULL) name = me;
  else ++name;

  printf(_("This is a program written by %s\n"),PACKAGE_BUGREPORT);
  printf("%s\n",_("This program is licenced with the GPL."));
  printf("%s\n",_("Use at your own risk!"));
  printf("%s\n\n",_("You need a phone made by Siemens to get an useful result."));
  printf("%s: %s",_("Syntax"),name);
  for (; types[i].name != NULL; ++i)
    if (types[i].required) printf(" %s",_(types[i].name));
    else printf(" [%s]",_(types[i].name));
  printf(" [%s...]\n",_("files"));

  for (k = 0; params[k].long_name || params[k].short_name; ++k) {
    i = str_width(_(params[k].long_name));
    if (params[k].pname) i += str_width(_(params[k].pname));
    if (total < i) total = i;
  }

  for (i = 0; types[i].name; ++i) {
    name = _(types[i].name);
    printf("\n%c%s:\n",toupper((int)name[0]),name+1);
    for (k = 0; params[k].long_name || params[k].short_name; ++k) {
      if (types[i].id == params[k].type_id)
	help_print_line(params[k].long_name,_(params[k].pname),total,
			params[k].short_name,_(params[k].descr));
    }
  }
  printf("\n%s\n",_("See man page for more detailed information."));
}

void version () {
  printf("%s\n",PACKAGE_VERSION);
}
